#!/usr/bin/perl 
##########################################################################
#                                                                        #
# Perl Script for SIP Registeration in location table of portaone Master #
# Switzernet(c)2011                                                      #
#                                                                        #
##########################################################################
# my modules
use strict;
use Asterisk::Manager;
use Sys::Syslog;
use POSIX;
use Config::IniFiles;
use Switch;
use warnings;
use DBI;
use threads;
use threads::shared;
##########################################################################
# Setting
my $lock_file = '/var/run/ast-reg-switz.pid';
my $lock_control = '/var/run/ast-control.pid';
my $runas_user = 'root';
my $monitor_dir = '/var/spool/asterisk/monitor';
my $config_dir = '/etc/astrad-V2/config';
##########################################################################
# Globals
my ($ast_hostname,$ast_username,$ast_password);
my ($reg_serv, $reg_sec, $nas_ip);
my ($DB_NAME_Reg,$DB_HOST_Reg,$DB_USER_Reg,$DB_PASS_Reg,$dbh_Reg,$Reg_connected);
my ($CustomerType,$lconf,$astman,$ast_connected);
my ($pid,$thr) ;
my $t_event : shared;
##########################################################################
#                             Initialization                             #
##########################################################################
# Check if already running
if( -e $lock_file ) {
    open(PID,$lock_file);
    my $pid=<PID>;
    close PID;
    chomp $pid;
    if( !-e "/proc/$pid" ) {
        print STDERR "Lock file present, but no process with pid=$pid.\n";
        die "Can't delete lock file $lock_file\n" if !unlink $lock_file;
        print STDERR "Lock file has been removed.\n";
    } else {
        die "Lockfile present, another copy is punning pid=$pid\n";
    }
}
my ($name, $passwd, $uid, $gid) = getpwnam($runas_user) or die "$runas_user not in passwd file";;
##########################################################################
# set C locale
POSIX::setlocale(LC_ALL, "C");
##########################################################################
# Become daemon
if( !defined($pid = fork()) ) 
    {
    die "cannot fork: $!";
    } 
    elsif ($pid) 
        {
        # Create lockfile, and finish parent process
        open(PID, "> $lock_file") || die "ast-reg-switz.pl: Unable to create lockfile $lock_file\n";
        print PID "$pid";
        close PID;
        chown $uid, $gid, $lock_file;
        exit;
        } 
        else 
            {
            # daemon
            setpgrp();
            select(STDERR); $| = 1;
            select(STDOUT); $| = 1;
            openlog('ast-reg-switz', 'cons,pid', 'daemon');
            syslog('notice', "Ast-Reg: Regisration SIP for Asterisk started");
            }
##########################################################################
# Install signall handler
$SIG{INT } = \&safe_exit;
$SIG{QUIT} = \&safe_exit;
$SIG{STOP} = \&safe_exit;
$SIG{TERM} = \&safe_exit;
$SIG{USR1} = \&safe_restart;
$SIG{HUP}  = \&load_config;
##########################################################################
# Drop privileges
setuid($uid);
$< = $uid;
$> = $uid;
##########################################################################
#                                FUNCTION                                #
##########################################################################
# Signal Handlers
sub safe_exit {
    my($sig) = @_;
    syslog('crit', "Ast-Reg: Caught a SIG$sig - shutting down");
    $astman->disconnect if $ast_connected;  
    $dbh_Reg->disconnect if $Reg_connected;    
    unlink $lock_file or syslog('crit', "Ast-Reg: Unable to create lockfile $lock_file\n");
    closelog();
    exit;
}
##########################################################################
# Load config
sub load_config {
    my $conf=Config::IniFiles->new(-file => "$config_dir/switzer.conf");
    return 0 if !defined $conf ;
    $nas_ip       = $conf->val('GLOBAL','NAS_IP');    
    $DB_NAME_Reg  = $conf->val('MASTER_DB','DB_NAME');
    $DB_HOST_Reg  = $conf->val('MASTER_DB','DB_HOST');
    $DB_USER_Reg  = $conf->val('MASTER_DB','DB_USER');
    $DB_PASS_Reg  = $conf->val('MASTER_DB','DB_PASS');    
    $ast_hostname = $conf->val('Manager','AMI_PERMIT');
    $ast_username = $conf->val('Manager','AMI_USER');
    $ast_password = $conf->val('Manager','AMI_SECRET');
    return 1;
}
##########################################################################
# sample '09:16:05 GMT Sat Dec 11 2004'
sub format_date_gmt {
    my ($date) = @_;
    return strftime "%H:%M:%S GMT %a %b %e %Y", gmtime($date);
}

##########################################################################
# sample '2004-12-11 09:16:05'
sub format_date_sql {
    my ($date) = @_;
    if ($date eq 0) {
        return "NULL";
    } else {
        return strftime("%Y-%m-%d %H:%M:%S", localtime($date));
    }
}
##########################################################################
# Astrad Reg. DB connect 
sub Reg_connect {
my $cop=0;
$Reg_connected = 1;
while( 1 ) {
    $cop++;
    $dbh_Reg = DBI->connect_cached("DBI:mysql:$DB_NAME_Reg;host=$DB_HOST_Reg", $DB_USER_Reg, $DB_PASS_Reg);
    if( $dbh_Reg ) {
        $Reg_connected = 1;
        $cop=0;
        syslog('info', 'Ast-Reg: Connected to Astrad Reg. DB!');
        return 1;
        } else {
            if ($Reg_connected) {
                syslog('err', 'Ast-Reg: Could not connect to Astrad Reg. DB!');
                $cop=1;
                }
            $Reg_connected = 0;
            syslog('info', "Ast-Reg: Connection to Astrad Reg. DB -> SLEEP $cop");
            $cop++ if $cop;
            return 0 if ($cop>=3);
            }
        sleep 1;
    }
}
##########################################################################
# Insert into location Table 
sub Insert_Reg {
    my $request=$_[0];
    return 0 if (!Reg_connect() || !defined $request);
    my $sth = $dbh_Reg->prepare($request);
    if ($sth->execute()) {
        $sth->finish;
        syslog('info', "Ast-Reg: Request = $request");
        return 1;
        }
return 0;
}
##########################################################################
# thread control
sub AMI_control {
syslog('info', "Ast-Reg: Start Thread AMI Control");
my ($l_event,$cop) = (0,0);
while ( 1 ) {
    $l_event = time();
    syslog('info', "Ast-Reg: Thread: Time Events -> Last = $t_event | Now = $l_event");
    if ($l_event - $t_event > 400) {
        syslog('crit', "Ast-Reg: Thread: Asterisk PBX is probably restarted !");
        safe_exit();
        }
    $cop=0 if ($l_event - $t_event < 4);
    $cop++;
    syslog('info', "Ast-Reg: Thread: AMI Control SLEEP $cop");
    sleep 2;
    }
}
##########################################################################
# AMI connect
sub AMI_connect {
$astman = new Asterisk::Manager;
$astman->user($ast_username);
$astman->secret($ast_password);
$astman->host($ast_hostname);
$t_event = time();
my $cop=0;
$ast_connected = 1;
while( 1 ) {
    $cop++;
    if( $astman->connect ) {
        $ast_connected = 1;
        $cop=0;
        syslog('info', 'Ast-Reg: Connected to Asterisk Manager !');
        $thr = threads->new(\&AMI_control);
        $astman->setcallback('DEFAULT', \&status_callback);
        eval { $astman->eventloop; };
        } 
    else { 
        if ($ast_connected) {
            syslog('err', 'Ast-Reg: Could not connect to Asterisk Manager!');
            $cop=1;
            }
        $ast_connected = 0;
        syslog('info', "Ast-Reg: AMI connection SLEEP $cop");
        $cop++ if $cop;
        return 0 if ($cop>=3);
        }
    sleep 1;
    }
}
##########################################################################
# Do Event
sub status_callback {
my (%event) = @_;
return  unless scalar(%event);

if ($event{'Event'} eq 'PeerStatus' && defined($event{'Peer'})) {
    my $request;
    my ($acc) = $event{'Peer'} =~ /^SIP\/(\d{11})/i;
    my $PStatus = $event{'PeerStatus'};
    my %peerinfo=$astman->sendcommand(Action => 'SIPshowpeer', Peer => $acc);
    if (defined($peerinfo{'Response'}) && $peerinfo{'Response'} eq 'Success')    
        {
        my $request;
        if ($PStatus =~ /^Registered/i && $peerinfo{'Address-IP'} && $peerinfo{'Address-Port'}) {
            my ($pubcon,$astcon,$ua)=($peerinfo{'Address-IP'}.":".$peerinfo{'Address-Port'},$peerinfo{'Reg-Contact '},$peerinfo{'SIP-Useragent'});
            my $exp=time();
            $exp+=$1 if $peerinfo{'RegExpire'} =~ /\s*(\d+) seconds/;
            $exp=format_date_sql($exp);
            my $offSet  = 1 + index($astcon, '@');
            my $localip = substr $astcon, $offSet, index($astcon, ':',$offSet) - $offSet;
            $request="delete from location where username='$acc' AND i_env=1 AND domain='$nas_ip'";
            return 0 if !Insert_Reg($request);
            $request="replace into location (username,contact,expires,q,callid,cseq,flags,user_agent,received,i_env,domain) values ('$acc', 'sip:$acc\@$pubcon', '$exp', -1.00, 'asterisk\@$localip', 1, 1, '$ua', 'NULL', 1, '$nas_ip')";
            }
        elsif ($PStatus =~ /^Unregistered/i && !$peerinfo{'Address-Port'}) {
            $request="delete from location where username='$acc' AND i_env=1 AND domain='$nas_ip'";
            }
        return 0 if !Insert_Reg($request);
        syslog('info', "Ast-Reg: $acc : $PStatus");
        }
    $t_event = time();
    }
if ($event{'Event'}  =~ /Shutdown/i ) {
    syslog('info', "Ast-Reg: Asterisk Shutdown");
    safe_exit();
    }
if ($event{'Event'} =~ /Reload/i ) {
    syslog('info', "Ast-Reg: Asterisk Reload");
    $t_event = time();
    load_config();    
    }
}
##########################################################################
#                                 main()                                 #
##########################################################################
$lconf=0;
syslog('info', "Ast-Reg: START Registration Perl Script");
while ( 1 ) {
    if (!$lconf && !load_config()) {
        syslog('crit', "Ast-Reg: Config file error!");
        next;
        }
    $lconf=1;
    if (!AMI_connect()) {
        syslog('crit', "Ast-Reg: Error in Astrerisk Manager connection");
        $lconf=0;
        sleep 1; 
        }
    }
##########################################################################
#                                  END                                   #
##########################################################################
